@@ -9,6 +9,8 @@ and the messages, then replies to the SMS by saying "thank you"
 
 from __future__ import print_function
 
+import logging
+
 PORT = '/dev/ttyUSB2'
 BAUDRATE = 115200
 
@@ -21,10 +23,15 @@ def handleSms(sms):
     print('SMS sent.\n')
     
 def main():
+    print('Initializing modem...')
+    #logging.basicConfig(format='%(levelname)s: %(message)s', level=logging.DEBUG)
     modem = GsmModem(PORT, BAUDRATE, smsReceivedCallbackFunc=handleSms)
     modem.connect()
-    print('Waiting for SMS message...')    
-    modem.rxThread.join(2**31) # Specify a (huge) timeout so that it essentially blocks indefinitely, but still receives CTRL+C interrupt signal
+    print('Waiting for SMS message...')
+    try:    
+        modem.rxThread.join(2**31) # Specify a (huge) timeout so that it essentially blocks indefinitely, but still receives CTRL+C interrupt signal
+    finally:
+        modem.close();
 
 if __name__ == '__main__':
-    main()
\ No newline at end of file
+    main()
